/*
 * Kneron ssd_fd_lm Companion Application
 *
 * Copyright (C) 2020 Kneron, Inc. All rights reserved.
 *
 */

#include <stdio.h>
#include <string.h>
#include "base.h"
#include "kmdw_console.h"
#include "kapp_ops.h"
#include "ssd_fd_lm_ops.h"
#include "kmdw_model.h"
#include "kmdw_memory.h"

#define TY_MAX_IMAGE_BUFS           4
#define TY_MAX_BOX_NUM              50
#define IMG_FORMAT_AGE_RGB         (NPU_FORMAT_RGB565 )
#define ty_msg(fmt, ...) info_msg(fmt, ##__VA_ARGS__)

typedef struct ssd_fd_lm_result_s_l {
    uint32_t class_count;            // total class count
    uint32_t box_count;              // boxes of all classes
    struct bounding_box_s boxes[TY_MAX_BOX_NUM];  // box_count
} ssd_fd_lm_result_s_l_t;

#define FACE_SCORE_THRESHOLD  0.5f
static struct ssd_fd_lm_result_s_l* ssd_res;
static struct landmark_result_s * output;
const float ssd_nms_thresh = 0.5;
static struct facedet_result_s* result_p;

static int32_t model_app_fdssd_det(struct ssd_fd_lm_result_s_l *out, struct kdp_img_cfg *img_cfg,
                                   enum model_type model_type, uint32_t dme_mode)
{   
    memset(out, 0, sizeof(struct ssd_fd_lm_result_s_l));
    kmdw_model_config_img(img_cfg, NULL, NULL, NULL);
    kmdw_model_config_result(0, 0);
    int status = kmdw_model_run("model_app_fdssd_det", out, model_type, false);
    if (status == KMDW_MODEL_RUN_RC_ABORT) {
        info_msg("[INFO] Got abort request\n");
        return KAPP_ABORT;
    } else if(status == KMDW_MODEL_RUN_RC_ERROR) {
        return KAPP_ERR;
    } else if(status != IMAGE_STATE_DONE) {
        return KAPP_ERR;
    }

    return KAPP_OP_OK;
}

static int32_t model_app_lm(struct landmark_result_s *lm_out,
                                    struct kdp_img_cfg *img_cfg,
                                    struct kdp_crop_box_s *crop_bbox,
                                    enum model_type model_type)
{
    kmdw_model_config_img(img_cfg, crop_bbox, NULL, NULL);
    kmdw_model_config_result(0, 0);
    int status = kmdw_model_run("model_app_lm", lm_out, model_type, false);

    if (status == KMDW_MODEL_RUN_RC_ABORT) {
        return status;
    } else if(status == KMDW_MODEL_RUN_RC_ERROR) {
        return status;
    } else if(status != IMAGE_STATE_DONE) {
        return status;
    } else {
        for(int index=0; index < LAND_MARK_POINTS; index++){
            if (lm_out->marks[index].x <= 0 || lm_out->marks[index].x > img_cfg->image_col 
                || lm_out->marks[index].y <= 0 || lm_out->marks[index].y > img_cfg->image_row || lm_out->score < (float)0.95){
                    return KAPP_ERR;
            }
        }
        return KAPP_OP_OK;
    }
}

static float overlap(float l1, float r1, float l2, float r2)
{
    float left = l1 > l2 ? l1 : l2;
    float right = r1 < r2 ? r1 : r2;
    return right - left;
}

static float box_intersection(struct bounding_box_s *a, struct bounding_box_s *b)
{
    float w, h, area;

    w = overlap(a->x1, a->x2, b->x1, b->x2);
    h = overlap(a->y1, a->y2, b->y1, b->y2);

    if (w < 0 || h < 0)
        return 0;

    area = w * h;
    return area;
}

static float box_union(struct bounding_box_s *a, struct bounding_box_s *b)
{
    float i, u;

    i = box_intersection(a, b);
    u = (a->y2 - a->y1) * (a->x2 - a->x1) + (b->y2 - b->y1) * (b->x2 - b->x1) - i;

    return u;
}

static float box_iou(struct bounding_box_s *a, struct bounding_box_s *b)
{
    float c;

    if (box_intersection(a, b) / box_intersection(a, a) > box_intersection(a, b) / box_intersection(b, b))
        c = box_intersection(a, b) / box_intersection(a, a);
    else
        c = box_intersection(a, b) / box_intersection(b, b);
    c = 0.0;
    if (c < box_intersection(a, b) / box_union(a, b))
        c = box_intersection(a, b) / box_union(a, b);

    return c;
}

static struct kdp_crop_box_s get_crop_bbox_lm(const struct kdp_img_cfg *img_cfg,
                                              const struct facedet_result_s *fd_bbox)
{
    int img_idx = img_cfg->image_buf_active_index;
    struct kdp_img_raw_s *raw_img = kmdw_model_get_raw_img(img_idx);
    struct kdp_crop_box_s bbox;
    bbox.top    = fd_bbox->xywh[1];
    bbox.bottom = raw_img->input_row - fd_bbox->xywh[1] - fd_bbox->xywh[3];
    bbox.left   = fd_bbox->xywh[0];
    bbox.right  = raw_img->input_col - fd_bbox->xywh[0] - fd_bbox->xywh[2];

    return bbox;
}

static int ssd_fd_lm_start(uint32_t app_id)
{
    if (app_id != APP_FD_LM)
        return KAPP_ERR;

    return KAPP_OP_OK;
}

static int ssd_fd_lm_run_image(uint32_t app_id, struct kapp_img_run_s *img_run_p)
{
    if (img_run_p == NULL)
        return -1;
        
    int32_t ret = model_app_fdssd_det(ssd_res, &img_run_p->img_cfg, KNERON_FD_MASK_MBSSD_200_200_3, 1);
    if (ret != KAPP_OP_OK) {
        return ret;
    }

    for (int i = 0; i < ssd_res->box_count; i++) {
        info_msg("[SCPU] x1: %f, y1: %f, x2: %f, y2: %f\n", ssd_res->boxes[i].x1, ssd_res->boxes[i].y1, ssd_res->boxes[i].x2, ssd_res->boxes[i].y2);
        if (ssd_res->boxes[i].score < FACE_SCORE_THRESHOLD) {
            ssd_res->boxes[i].score = 0;
            continue;
        }

        for(int j=i+1; j < ssd_res->box_count; j++){
            if (ssd_res->boxes[j].score <= 0){
                continue;
            }
            if(box_iou(&ssd_res->boxes[i], &ssd_res->boxes[j]) > ssd_nms_thresh){
                if(ssd_res->boxes[i].score <= ssd_res->boxes[j].score){
                    ssd_res->boxes[i].score = 0;
                    break;
                }else{
                    ssd_res->boxes[j].score = 0;
                    continue;
                }
            }
        }

    }

    for (int i = 0; i < ssd_res->box_count; i++) {
        if (ssd_res->boxes[i].score > 0) { 
            result_p->xywh[0] = ssd_res->boxes[i].x1;
            result_p->xywh[1] = ssd_res->boxes[i].y1;
            result_p->xywh[2] = ssd_res->boxes[i].x2 - result_p->xywh[0];
            result_p->xywh[3] = ssd_res->boxes[i].y2 - result_p->xywh[1];
            struct kdp_crop_box_s crop_box = get_crop_bbox_lm(&img_run_p->img_cfg, result_p);

            ret = model_app_lm(output, &img_run_p->img_cfg, &crop_box, KNERON_LM_5PTS_ONET_56_56_3);
            
            if (ret != KAPP_OP_OK) {
                ssd_res->boxes[i].score = 0;
            }						
        }
    }

    int det_size = ssd_res->box_count * sizeof(struct bounding_box_s) + sizeof(uint32_t)*2;
    memcpy(img_run_p->out, ssd_res, det_size);

    return KAPP_OP_OK;
}

static int ssd_fd_lm_get_result_len(uint32_t app_id, void *result_buf)
{
    ssd_fd_lm_result_s_l_t *out;
    int len;

    out = (ssd_fd_lm_result_s_l_t *)result_buf;
    len = out->box_count * sizeof(struct bounding_box_s) + sizeof(uint32_t) * 2;
    return len;
}

static struct kapp_ops kapp_ssd_fd_lm_ops = {
    .start          = ssd_fd_lm_start,
    .run_image      = ssd_fd_lm_run_image,
    .get_result_len = ssd_fd_lm_get_result_len,
};

struct kapp_ops *ssd_fd_lm_get_ops(void)
{
    ssd_res = (struct ssd_fd_lm_result_s_l*)kmdw_ddr_reserve(sizeof(struct ssd_fd_lm_result_s_l));
    output= (struct landmark_result_s *)kmdw_ddr_reserve(sizeof(struct landmark_result_s));	
    result_p = (struct facedet_result_s*)kmdw_ddr_reserve(sizeof(struct facedet_result_s));
    return &kapp_ssd_fd_lm_ops;
}
